import logging
from .Discord import Discord
from .... import REDIRECT_URI_NAME
from django.shortcuts import redirect
from django.views.generic import View
from .models import Discord as DiscordModel
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required


@method_decorator(decorator=login_required, name='dispatch')
class Authenticate(View):
    def get(self, request):
        discord = Discord()
        return_url = discord.build_url(request, '__account_discord_authenticated')
        discord_login_url = discord.auth_url(request, return_url)
        return redirect(discord_login_url)


@method_decorator(decorator=login_required, name='dispatch')
class Authenticated(View):

    def get(self, request):
        discord = Discord()
        redirect_url = discord.build_url(request, '__account_discord_authenticated')
        payload = discord.verify_auth(request, redirect_url)
        self.save_connection(request, payload)
        return redirect(REDIRECT_URI_NAME)

    def save_connection(self, request, payload):
        try:
            if payload.get('access_token') and payload.get('refresh_token'):
                data = {
                    'user': request.user,
                    'access_token': payload.get('access_token'),
                    'expires_in': payload.get('expires_in'),
                    'refresh_token': payload.get('refresh_token', None),
                    'token_type': payload.get('token_type')
                }
                DiscordModel.objects.update_or_create(user=request.user, defaults=data)
        except Exception as e:
            logging.error(str(e))


@method_decorator(decorator=login_required, name='dispatch')
class UnAuthenticate(View):
    def get(self, request):
        discord = Discord()
        discord.revoke_token(request.user)
        return redirect(REDIRECT_URI_NAME)



from . import views
from django.urls import path

profile_urlpatterns = [
    path('', views.Index.as_view(), name='__account_profile_index'),
    path('profile-edit', views.Edit.as_view(), name='__account_profile_edit'),
]



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _


class Profile(models.Model):
    MALE = 'MA'
    FEMALE = 'FM'
    OTHER = 'OT'
    genders = [
        (MALE, _('male')),
        (FEMALE, _('female')),
        (OTHER, _('other'))
    ]
    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    image_url = models.URLField(null=True, blank=True, db_comment='Profile image url of user.')
    gender = models.CharField(max_length=3, choices=genders, null=True, default=None)
    date_of_birth = models.DateField(null=True, blank=True)
    phone_number = models.CharField(max_length=20, db_comment='Include with country code.', null=True)

    def __str__(self):
        return f'{self.user.username}'



from . import service_template
from django.urls import reverse_lazy
from django.http import JsonResponse
from django.shortcuts import redirect
from ...utils.forms import EditProfileForm
from django.contrib.auth import get_user_model
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView, FormView
from django.contrib.auth.decorators import login_required
from ...models import Profile, Google, Steam, Twitch, Discord


@method_decorator(decorator=login_required, name='dispatch')
class Index(TemplateView):
    template_name = f'{service_template}/index.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data()

        google = Google.objects.filter(user=self.request.user)
        steam = Steam.objects.filter(user=self.request.user)
        twitch = Twitch.objects.filter(user=self.request.user)
        discord = Discord.objects.filter(user=self.request.user)
        context.update({
            'edit_form': EditProfileForm,
            'google': google,
            'steam': steam,
            'twitch': twitch,
            'discord': discord,
        })

        return context


@method_decorator(decorator=login_required, name='dispatch')
class Edit(FormView):
    success_url = reverse_lazy('__account_profile_index')
    form_class = EditProfileForm

    def get(self, request, *args, **kwargs):
        return redirect(self.success_url)

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        context['edit_form'] = EditProfileForm
        return context

    def form_invalid(self, form):
        return JsonResponse({'status': False, 'errors': form.errors}, status=422)

    def form_valid(self, form):
        profile_data = form.cleaned_data
        obj, created = Profile.objects.update_or_create(user=self.request.user, defaults={
            'gender': profile_data.get('gender'),
            'date_of_birth': profile_data.get('date_of_birth'),
            'phone_number': profile_data.get('phone_number')
        })
        if obj:
            get_user_model().objects.filter(pk=self.request.user.pk).update(
                first_name=profile_data.get('first_name'),
                last_name=profile_data.get('last_name')
            )
        return JsonResponse({'status': True, 'url': self.success_url}, status=200)
